@@ -6,7 +6,7 @@ import System.Environment
 import Text.XHtml.Transitional
 import System.Console.GetOpt
 import System.Exit
-import System.FilePath (takeFileName, takeExtension)
+import System.FilePath (takeFileName)
 import Data.Maybe (listToMaybe)
 import Data.Char (toLower)
 
@@ -16,6 +16,7 @@ data Flag = CssPath String
           | List
           | NumberLines
           | Syntax String
+          | Detailed
           | TitleAttributes
           | Version
           deriving (Eq, Show)
@@ -28,6 +29,7 @@ options =
   , Option ['l'] ["list"] (NoArg List)   "list available language syntaxes"
   , Option ['n'] ["number-lines"] (NoArg NumberLines)  "number lines"
   , Option ['s'] ["syntax"] (ReqArg Syntax "SYNTAX")  "specify language syntax to use"
+  , Option ['d'] ["details"] (NoArg Detailed) "include detailed lexical information in classes"
   , Option ['t'] ["title-attributes"] (NoArg TitleAttributes)  "include structure in title attributes"
   , Option ['v'] ["version"] (NoArg Version)   "print version"
   ]
@@ -81,10 +83,9 @@ main = do
              else mapM readFile fnames >>= return . filterNewlines . concat
   let lang' = case syntaxOf opts of
                     Just e   -> Just e
-                    Nothing  -> if null fnames
-                                   then Nothing
-                                   else let firstExt = drop 1 $ takeExtension $ head fnames
-                                        in  listToMaybe $ languagesByExtension firstExt
+                    Nothing  -> case fnames of
+                                     []     -> Nothing
+                                     (x:_)  -> listToMaybe $ languagesByFilename $ takeFileName x
   lang <- if lang' == Nothing
              then hPutStrLn stderr "No syntax specified." >>
                   hPutStrLn stderr (usageInfo usageHeader options) >>
@@ -94,8 +95,10 @@ main = do
   if not (lang `elem` (map (map toLower) languages))
      then hPutStrLn stderr ("Unknown syntax: " ++ lang) >> exitWith (ExitFailure 4)
      else return ()
-  let highlightOpts = (if TitleAttributes `elem` opts then [OptTitleAttributes] else []) ++
-                      (if NumberLines `elem` opts then [OptNumberLines, OptLineAnchors] else [])
+  let highlightOpts = [OptTitleAttributes | TitleAttributes `elem` opts] ++
+                      [OptDetailed | Detailed `elem` opts] ++
+                      [OptNumberLines | NumberLines `elem` opts] ++
+                      [OptLineAnchors | NumberLines `elem` opts]
   let css = case cssPathOf opts of
                    Nothing      -> style ! [thetype "text/css"] $ primHtml defaultHighlightingCss 
                    Just cssPath -> thelink ! [thetype "text/css", href cssPath, rel "stylesheet"] << noHtml
